package parser

// === WILDCARDS ===
//
// The star       '*'  -> matches 0 or 1 slash-bounded string
// The multi star '**' -> matches 0 or more slash-separated strings
// The dot        '.'  -> matches 1 slash-bounded string
// The multi dot  '..' -> matches 1 or more slash-separated strings
//
// === SCHEME POLICY ===
//
// - The last '/' is optional
// - Any '**' at the very end will match anything that starts with the given prefix
//
// === LIMITATIONS ==
//
// - A scheme must begin with '/'
// - A scheme cannot contain something else than a STRING or WILDCARD between 2 '/' separators
// - A scheme STRING cannot contain the symbols '/' as a character
// - A scheme STRING containing '*' or '.' characters will be treating as STRING only
// - A maximum of 16 slash-separated matchers (STRING or WILDCARD) are allowed

const maxMatch = 16

// Represents an URI matcher
type matcher struct {
	pat string // pattern to match (empty if wildcard)
	req bool   // whether it is required
	mul bool   // whether multiple matches are allowed

	buf []string // matched content (when matching)
}

// Represents an URI scheme
type Scheme []*matcher
